package com.macro.mall.tiny.modules.business.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import com.macro.mall.tiny.common.exception.ApiException;
import com.macro.mall.tiny.common.exception.Asserts;
import com.macro.mall.tiny.common.util.CommonUtils;
import com.macro.mall.tiny.config.UploadConfig;
import com.macro.mall.tiny.domain.AdminUserDetails;
import com.macro.mall.tiny.modules.business.enums.DirectoryEnum;
import com.macro.mall.tiny.modules.business.model.*;
import com.macro.mall.tiny.modules.business.service.*;
import com.macro.mall.tiny.modules.ums.model.UmsAdmin;
import com.macro.mall.tiny.modules.ums.service.UmsAdminService;
import com.macro.mall.tiny.security.util.AuthUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.ArchiveInputStream;
import org.apache.commons.compress.archivers.zip.ZipArchiveInputStream;
import org.apache.commons.compress.utils.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.io.*;
import java.math.BigDecimal;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipInputStream;

/**
 * @author gjs
 * @description
 * @date 2021/3/28 17:44
 */
@Component
@Slf4j
public class AsyncHandler {

    @Autowired
    private UmsAdminService adminService;

    @Autowired
    private BProvinceServiceImpl provinceService;

    @Autowired
    private BLineService lineService;

    @Autowired
    private BLineUploadLogService lineUploadLogService;

    @Autowired
    private BTowerService towerService;

    @Autowired
    private BLineCacheService lineCacheService;

    @Autowired
    private BDirectoryCacheService directoryCacheService;

    @Async
    @Transactional
    public void handleDocument(Long provinceId, String fileSource, String lineName, AdminUserDetails userDetails, String charset) {
        StringBuilder warnMsg = new StringBuilder();
        BLine line = null;
        Long userId = null;
        try {
            if (provinceId == null) Asserts.fail("上传失败，省份不能为空，provinceId：" + provinceId);
            if (userDetails == null) Asserts.fail("上传失败，未获取到当前用户");

            BProvince province = provinceService.getById(provinceId);
            String provinceName = province.getProvince();

            String username = userDetails.getUsername();
            UmsAdmin umsAdmin = adminService.getAdminByUsername(username);
            userId = umsAdmin.getId();
            boolean canCover = AuthUtil.checkAccess(userDetails, AuthUtil.COVER);
            // 查看是否为线路创建者
            QueryWrapper<BLine> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(BLine::getName, lineName);
            line = lineService.getOne(wrapper);
            List<BTower> towerList = Lists.newArrayList();
            if (line == null) {
                // 如果为空，则创建新线路
                BLine newLine = new BLine();
                newLine.setCreatorId(userId);
                newLine.setName(lineName);
                newLine.setProvinceId(provinceId);
                newLine.setCreateTime(new Date());
                lineService.save(newLine);
                line = newLine;

                canCover = true;
            } else if (line.getCreatorId().equals(userId)) {
                canCover = true;
            }

            // 先获取根目录
            Path rootPath = Paths.get(UploadConfig.powerPath + provinceName);
            if (!Files.exists(rootPath)) {
                Files.createDirectories(rootPath);
            }

            // 获取枚举类判断
            DirectoryEnum[] enums = DirectoryEnum.values();
            HashMap<String, Boolean> booleanHashMap = new HashMap<>();
            for (DirectoryEnum directory : enums) {
                booleanHashMap.put(directory.getName(), false);
            }

            try (InputStream fis = Files.newInputStream(Paths.get(fileSource));
                 InputStream bis = new BufferedInputStream(fis);
                 ArchiveInputStream ais = new ZipArchiveInputStream(bis, charset)) {

                ArchiveEntry entry;
                while (Objects.nonNull(entry = ais.getNextEntry())) {
                    if (!ais.canReadEntryData(entry)) {
                        continue;
                    }

                    String entryName = entry.getName();
                    // 解析各项路径
                    String[] pathStr = entryName.split(UploadConfig.SEPARATOR);
                    if (pathStr.length < 2) {
                        continue;
                    }
                    entryName = lineName + entryName.substring(entryName.indexOf(UploadConfig.SEPARATOR));
                    String SecDirectoryName = pathStr[1].charAt(0) == UploadConfig.AUTH_STR ? pathStr[1].substring(1) : pathStr[1];
                    if (booleanHashMap.get(SecDirectoryName) != null) {
                        booleanHashMap.put(SecDirectoryName, true);
                    } else if ("杆塔信息录入.xlsx".equals(SecDirectoryName) || "杆塔信息录入.xls".equals(SecDirectoryName)) {
                        List<TowerExcelModel> modelArrayList = EasyExcel.read(ais, TowerExcelModel.class, null).sheet().doReadSync();
                        BLine finalLine = line;
                        towerList = modelArrayList.stream().map(model -> {
                            BTower tower = new BTower();
                            tower.setName(model.getName());
                            tower.setShape(model.getShape());
                            tower.setSort(model.getSort());
                            tower.setHardwareType(model.getHardware());
                            // 经纬度设置
                            String lonLat = model.getLonLat();
                            String[] lonLats = lonLat.split(",");
                            BigDecimal lon = CommonUtils.Dms2D(lonLats[0]);
                            BigDecimal lat = CommonUtils.Dms2D(lonLats[1]);
                            tower.setLon(lon);
                            tower.setLat(lat);
                            tower.setLineId(finalLine.getId());
                            tower.setHasFile(0);
                            tower.setCreateTime(new Date());
                            return tower;
                        }).collect(Collectors.toList());
//                            towerService.saveAndUpdate(towerList);
                        continue;
                    } else {
                        warnMsg.append(entryName).append("第二层目录解析出错;");
                        continue;
                    }

                    Path entryFile = rootPath.resolve(entryName);
                    if (entry.isDirectory()) {
                        if (!Files.isDirectory(entryFile)) {
                            Files.createDirectories(entryFile);
                        }
                    } else {
                        Path parent = entryFile.getParent();
                        if (!canCover && Files.exists(entryFile)) {
                            warnMsg.append(entryName).append(entryFile.getFileName().toString()).append("已存在，无权覆盖;");
                        } else {
                            Files.createDirectories(parent);
                            try (OutputStream o = Files.newOutputStream(entryFile)) {
                                IOUtils.copy(ais, o);
                            }
                        }
                    }
                }
            }
            booleanHashMap.forEach((key, val) -> {
                if (!val) {
                    warnMsg.append("缺少").append(key).append("文件夹;");
                }
            });
            // 存储上传记录
            BLineUploadLog lineUploadLog = new BLineUploadLog();
            lineUploadLog.setLineId(line.getId());
            lineUploadLog.setStatus(1);
            lineUploadLog.setUploaderId(userId);
            lineUploadLog.setUploadTime(new Date());
            lineUploadLog.setWarnMsg(warnMsg.toString());
            lineUploadLogService.save(lineUploadLog);

            // 后续更新塔是否包含资料
            QueryWrapper<BTower> towerWrapper = new QueryWrapper<>();
            towerWrapper.lambda().eq(BTower::getLineId, line.getId());
            List<BTower> oriTowerList = towerService.list(towerWrapper);
            HashSet<BTower> towerSet = new HashSet<>(oriTowerList);
            towerSet.addAll(towerList);
            towerService.setHasFile(new ArrayList<>(towerSet), userDetails);

            // 后续清楚缓存处理
            lineCacheService.delLineList();
            directoryCacheService.delDirectory();
        } catch (ApiException e) {
            log.error("上传文件抛出业务异常", e);
        } catch (Exception e) {
            log.error("上传文件抛出异常", e);
            if (line != null && userId != null) {
                // 存储上传记录
                BLineUploadLog lineUploadLog = new BLineUploadLog();
                lineUploadLog.setLineId(line.getId());
                lineUploadLog.setStatus(0);
                lineUploadLog.setUploaderId(userId);
                lineUploadLog.setUploadTime(new Date());
                lineUploadLog.setWarnMsg(e.getMessage());
                lineUploadLogService.save(lineUploadLog);
            }
        } finally {
            File file = new File(fileSource);
            if (file.exists()) {
                file.delete();
            }
        }
    }
}
